locals {
  redrive_policy = jsonencode(
    {
      deadLetterTargetArn = var.use_sqs_as_dlq ? "" : join("", aws_sqs_queue.deadletter_sqs_queue.*.arn)
      maxReceiveCount     = var.max_receive_count
    }
  )
  allowed_aws_services_for_sqs_send_message = split(",", var.allowed_aws_services_for_sqs_send_message)
}

#SQS Dead Letter Queue
resource "aws_sqs_queue" "deadletter_sqs_queue" {
  count                             = var.use_sqs_as_dlq ? 0 : 1
  name                              = var.dlq_name
  delay_seconds                     = var.delay_seconds
  fifo_queue                        = var.fifo_queue
  max_message_size                  = var.max_message_size
  visibility_timeout_seconds        = var.visibility_timeout_seconds
  message_retention_seconds         = var.message_retention_seconds
  receive_wait_time_seconds         = var.receive_wait_time_seconds
  tags                              = var.tags
  kms_master_key_id                 = var.kms_master_key_id
  kms_data_key_reuse_period_seconds = var.kms_data_key_reuse_period_seconds
}

#SQS QUEUE Resource without KMS encryption
resource "aws_sqs_queue" "sqs_queue" {
  name                              = var.sqs_queue_name
  delay_seconds                     = var.delay_seconds
  fifo_queue                        = var.fifo_queue
  content_based_deduplication       = var.content_based_deduplication
  max_message_size                  = var.max_message_size
  visibility_timeout_seconds        = var.visibility_timeout_seconds
  message_retention_seconds         = var.message_retention_seconds
  receive_wait_time_seconds         = var.receive_wait_time_seconds
  redrive_policy                    = var.use_sqs_as_dlq ? null : local.redrive_policy
  kms_master_key_id                 = var.kms_master_key_id
  kms_data_key_reuse_period_seconds = var.kms_data_key_reuse_period_seconds
  tags                              = var.tags
}

#SQS Policy
data "aws_iam_policy_document" "aws_sqs_queue_policy" {
  count     = length(var.allowed_aws_services_for_sqs_send_message) > 0 ? 1 : 0
  policy_id = "SQSPolicy"

  statement {
    sid       = "SQSAccessPolicy"
    effect    = "Allow"
    actions   = ["sqs:SendMessage"]
    resources = [aws_sqs_queue.sqs_queue.arn]

    principals {
      type        = "Service"
      identifiers = local.allowed_aws_services_for_sqs_send_message
    }
  }
}

#SQS Policy
resource "aws_sqs_queue_policy" "sqs_policy" {
  count     = length(var.allowed_aws_services_for_sqs_send_message) > 0 ? 1 : 0
  queue_url = aws_sqs_queue.sqs_queue.id
  policy    = data.aws_iam_policy_document.aws_sqs_queue_policy[0].json
}

resource "aws_iam_policy" "consumer-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-sqs-consumer"
  path        = "/"
  description = "SQS queue consumer policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "sqs:ReceiveMessage",
        "sqs:DeleteMessage",
        "sqs:ChangeMessageVisibility",
        "sqs:Get*"
      ],
      "Effect": "Allow",
      "Resource": ["${aws_sqs_queue.sqs_queue.arn}"]
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }
  tags = var.tags
}

resource "aws_iam_policy" "producer-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-sqs-producer"
  path        = "/"
  description = "SQS queue producer policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "sqs:SendMessage",
        "sqs:ChangeMessageVisibility",
        "sqs:Get*"
      ],
      "Effect": "Allow",
      "Resource": ["${aws_sqs_queue.sqs_queue.arn}"]
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }
  tags = var.tags
}
